@@ -3,4 +3,4 @@ from .client import APIClient
 from .connection import APIConnection, ConnectionParams
 from .core import MESSAGE_TYPE_TO_PROTO, APIConnectionError
 from .model import *
-from .util import resolve_ip_address, resolve_ip_address_getaddrinfo
+from .reconnect_logic import ReconnectLogic
