@@ -1,7 +1,6 @@
 import asyncio
 import logging
 from typing import (
-    TYPE_CHECKING,
     Any,
     Awaitable,
     Callable,
@@ -9,15 +8,14 @@ from typing import (
     List,
     Optional,
     Tuple,
+    Type,
     Union,
     cast,
 )
 
-import attr
-import zeroconf
 from google.protobuf import message
 
-from aioesphomeapi.api_pb2 import (  # type: ignore
+from .api_pb2 import (  # type: ignore
     BinarySensorStateResponse,
     CameraImageRequest,
     CameraImageResponse,
@@ -61,9 +59,10 @@ from aioesphomeapi.api_pb2 import (  # type: ignore
     SwitchStateResponse,
     TextSensorStateResponse,
 )
-from aioesphomeapi.connection import APIConnection, ConnectionParams
-from aioesphomeapi.core import APIConnectionError
-from aioesphomeapi.model import (
+from .connection import APIConnection, ConnectionParams
+from .core import APIConnectionError
+from .host_resolver import ZeroconfInstanceType
+from .model import (
     APIVersion,
     BinarySensorInfo,
     BinarySensorState,
@@ -79,6 +78,7 @@ from aioesphomeapi.model import (
     CoverState,
     DeviceInfo,
     EntityInfo,
+    EntityState,
     FanDirection,
     FanInfo,
     FanSpeed,
@@ -87,6 +87,7 @@ from aioesphomeapi.model import (
     LegacyCoverCommand,
     LightInfo,
     LightState,
+    LogLevel,
     NumberInfo,
     NumberState,
     SensorInfo,
@@ -96,13 +97,9 @@ from aioesphomeapi.model import (
     TextSensorInfo,
     TextSensorState,
     UserService,
-    UserServiceArg,
     UserServiceArgType,
 )
 
-if TYPE_CHECKING:
-    from aioesphomeapi.api_pb2 import LogLevel  # type: ignore
-
 _LOGGER = logging.getLogger(__name__)
 
 ExecuteServiceDataType = Dict[
@@ -110,17 +107,18 @@ ExecuteServiceDataType = Dict[
 ]
 
 
+# pylint: disable=too-many-public-methods
 class APIClient:
     def __init__(
         self,
         eventloop: asyncio.AbstractEventLoop,
         address: str,
         port: int,
-        password: str,
+        password: Optional[str],
         *,
         client_info: str = "aioesphomeapi",
         keepalive: float = 15.0,
-        zeroconf_instance: Optional[zeroconf.Zeroconf] = None
+        zeroconf_instance: ZeroconfInstanceType = None,
     ):
         self._params = ConnectionParams(
             eventloop=eventloop,
@@ -131,7 +129,18 @@ class APIClient:
             keepalive=keepalive,
             zeroconf_instance=zeroconf_instance,
         )
-        self._connection = None  # type: Optional[APIConnection]
+        self._connection: Optional[APIConnection] = None
+        self._cached_name: Optional[str] = None
+
+    @property
+    def address(self) -> str:
+        return self._params.address
+
+    @property
+    def _log_name(self) -> str:
+        if self._cached_name is not None:
+            return f"{self._cached_name} @ {self.address}"
+        return self.address
 
     async def connect(
         self,
@@ -139,7 +148,7 @@ class APIClient:
         login: bool = False,
     ) -> None:
         if self._connection is not None:
-            raise APIConnectionError("Already connected!")
+            raise APIConnectionError(f"Already connected to {self._log_name}!")
 
         connected = False
         stopped = False
@@ -155,6 +164,7 @@ class APIClient:
                 await on_stop()
 
         self._connection = APIConnection(self._params, _on_stop)
+        self._connection.log_name = self._log_name
 
         try:
             await self._connection.connect()
@@ -165,7 +175,9 @@ class APIClient:
             raise
         except Exception as e:
             await _on_stop()
-            raise APIConnectionError("Unexpected error while connecting: {}".format(e))
+            raise APIConnectionError(
+                f"Unexpected error while connecting to {self._log_name}: {e}"
+            ) from e
 
         connected = True
 
@@ -176,15 +188,15 @@ class APIClient:
 
     def _check_connected(self) -> None:
         if self._connection is None:
-            raise APIConnectionError("Not connected!")
+            raise APIConnectionError(f"Not connected to {self._log_name}!")
         if not self._connection.is_connected:
-            raise APIConnectionError("Connection not done!")
+            raise APIConnectionError(f"Connection not done for {self._log_name}!")
 
     def _check_authenticated(self) -> None:
         self._check_connected()
         assert self._connection is not None
         if not self._connection.is_authenticated:
-            raise APIConnectionError("Not authenticated!")
+            raise APIConnectionError(f"Not authenticated for {self._log_name}!")
 
     async def device_info(self) -> DeviceInfo:
         self._check_connected()
@@ -192,21 +204,16 @@ class APIClient:
         resp = await self._connection.send_message_await_response(
             DeviceInfoRequest(), DeviceInfoResponse
         )
-        return DeviceInfo(
-            uses_password=resp.uses_password,
-            name=resp.name,
-            mac_address=resp.mac_address,
-            esphome_version=resp.esphome_version,
-            compilation_time=resp.compilation_time,
-            model=resp.model,
-            has_deep_sleep=resp.has_deep_sleep,
-        )
+        info = DeviceInfo.from_pb(resp)
+        self._cached_name = info.name
+        self._connection.log_name = self._log_name
+        return info
 
     async def list_entities_services(
         self,
     ) -> Tuple[List[EntityInfo], List[UserService]]:
         self._check_authenticated()
-        response_types = {
+        response_types: Dict[Any, Optional[Type[EntityInfo]]] = {
             ListEntitiesBinarySensorResponse: BinarySensorInfo,
             ListEntitiesCoverResponse: CoverInfo,
             ListEntitiesFanResponse: FanInfo,
@@ -234,21 +241,7 @@ class APIClient:
         services: List[UserService] = []
         for msg in resp:
             if isinstance(msg, ListEntitiesServicesResponse):
-                args = []
-                for arg in msg.args:
-                    args.append(
-                        UserServiceArg(
-                            name=arg.name,
-                            type_=arg.type,
-                        )
-                    )
-                services.append(
-                    UserService(
-                        name=msg.name,
-                        key=msg.key,
-                        args=args,  # type: ignore
-                    )
-                )
+                services.append(UserService.from_pb(msg))
                 continue
             cls = None
             for resp_type, cls in response_types.items():
@@ -256,17 +249,14 @@ class APIClient:
                     break
             else:
                 continue
-            cls = cast(type, cls)
-            kwargs = {}
-            for key, _ in attr.fields_dict(cls).items():
-                kwargs[key] = getattr(msg, key)
-            entities.append(cls(**kwargs))
+            assert cls is not None
+            entities.append(cls.from_pb(msg))
         return entities, services
 
-    async def subscribe_states(self, on_state: Callable[[Any], None]) -> None:
+    async def subscribe_states(self, on_state: Callable[[EntityState], None]) -> None:
         self._check_authenticated()
 
-        response_types = {
+        response_types: Dict[Any, Type[EntityState]] = {
             BinarySensorStateResponse: BinarySensorState,
             CoverStateResponse: CoverState,
             FanStateResponse: FanState,
@@ -284,7 +274,7 @@ class APIClient:
             if isinstance(msg, CameraImageResponse):
                 data = image_stream.pop(msg.key, bytes()) + msg.data
                 if msg.done:
-                    on_state(CameraState(key=msg.key, image=data))
+                    on_state(CameraState.from_pb(msg))
                 else:
                     image_stream[msg.key] = data
                 return
@@ -295,11 +285,8 @@ class APIClient:
             else:
                 return
 
-            kwargs = {}
             # pylint: disable=undefined-loop-variable
-            for key, _ in attr.fields_dict(cls).items():
-                kwargs[key] = getattr(msg, key)
-            on_state(cls(**kwargs))
+            on_state(cls.from_pb(msg))
 
         assert self._connection is not None
         await self._connection.send_message_callback_response(
@@ -309,7 +296,7 @@ class APIClient:
     async def subscribe_logs(
         self,
         on_log: Callable[[SubscribeLogsResponse], None],
-        log_level: Optional["LogLevel"] = None,
+        log_level: Optional[LogLevel] = None,
     ) -> None:
         self._check_authenticated()
 
@@ -330,10 +317,7 @@ class APIClient:
 
         def on_msg(msg: message.Message) -> None:
             if isinstance(msg, HomeassistantServiceResponse):
-                kwargs = {}
-                for key, _ in attr.fields_dict(HomeassistantServiceCall).items():
-                    kwargs[key] = getattr(msg, key)
-                on_service_call(HomeassistantServiceCall(**kwargs))
+                on_service_call(HomeassistantServiceCall.from_pb(msg))
 
         assert self._connection is not None
         await self._connection.send_message_callback_response(
@@ -571,12 +555,12 @@ class APIClient:
                 UserServiceArgType.FLOAT_ARRAY: "float_array",
                 UserServiceArgType.STRING_ARRAY: "string_array",
             }
-            # pylint: disable=redefined-outer-name
-            if arg_desc.type_ in map_array:
-                attr = getattr(arg, map_array[arg_desc.type_])
+            if arg_desc.type in map_array:
+                attr = getattr(arg, map_array[arg_desc.type])
                 attr.extend(val)
             else:
-                setattr(arg, map_single[arg_desc.type_], val)
+                assert arg_desc.type in map_single
+                setattr(arg, map_single[arg_desc.type], val)
 
             args.append(arg)
         # pylint: disable=no-member
